## $Id: studycourse.py 16907 2022-03-25 18:49:53Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Container which holds the data of the student study courses
and contains the (student) study level objects.
"""
import grok
from zope.interface import implementedBy
from zope.component import createObject
from waeup.kofa.students.studycourse import StudentStudyCourseFactory
from waeup.kofa.utils.helpers import attrs_to_fields
from waeup.kofa.students.interfaces import (
    IStudentNavigation, IStudentStudyLevel)
from waeup.kofa.students.workflow import CLEARED, RETURNING, PAID
from kofacustom.nigeria.students.studycourse import NigeriaStudentStudyCourse
from waeup.aaue.students.interfaces import ICustomStudentStudyCourse

class CustomStudentStudyCourse(NigeriaStudentStudyCourse):
    """This is a container for study levels. All pg students
    must register courses. Thus all pg certificates are 'special'
    and the property method `ICertificate.check_pg_conditions`
    is not applied.
    """
    grok.implements(ICustomStudentStudyCourse, IStudentNavigation)
    grok.provides(ICustomStudentStudyCourse)

    @property
    def is_postgrad(self):
        if self.certificate is None:
            return False
        return self.certificate.study_mode.startswith('special_pg')

    @property
    def is_special_postgrad(self):
        return self.is_postgrad

    def addStudentStudyLevel(self, cert, studylevel, course_category=None):
        """Add a study level object.
        """
        if not IStudentStudyLevel.providedBy(studylevel):
            raise TypeError(
                'StudentStudyCourses contain only IStudentStudyLevel instances')
        self[str(studylevel.level)] = studylevel
        #studylevel.addCertCourseTickets(cert)
        # Collect carry-over courses in base levels (not in repeating levels)
        try:
            co_enabled = grok.getSite()['configuration'].carry_over
        except TypeError:
            # In tests we might not have a site object
            co_enabled = True
        if not co_enabled or studylevel.level % 100 != 0:
            return
        levels = sorted(self.keys())
        index = levels.index(str(studylevel.level))
        co_ticket = None
        if index > 0:
            previous_level = self[levels[index-1]]
            for key, val in previous_level.items():
                if val.total_score >= val.passmark:
                    continue
                if course_category and val.course_category != course_category:
                    continue
                if key in self[str(studylevel.level)]:
                    # Carry-over ticket exists
                    continue
                co_ticket = createObject(u'waeup.CourseTicket')
                for name in ['code', 'title', 'credits', 'passmark',
                             'semester', 'mandatory', 'fcode', 'dcode',
                             'course_category']:
                    setattr(co_ticket, name, getattr(val, name))
                co_ticket.automatic = True
                co_ticket.carry_over = True
                self[str(studylevel.level)][co_ticket.code] = co_ticket
        # AAUE modifaction: Add cert courses tickets only if no carry-over
        # ticket has been found
        if not co_ticket:
            studylevel.addCertCourseTickets(cert)
        return


CustomStudentStudyCourse = attrs_to_fields(CustomStudentStudyCourse)

class CustomStudentStudyCourseFactory(StudentStudyCourseFactory):
    """A factory for student study courses.
    """

    def __call__(self, *args, **kw):
        return CustomStudentStudyCourse()

    def getInterfaces(self):
        return implementedBy(CustomStudentStudyCourse)
